#ifndef _BISHOP_H
#define _BISHOP_H

#include <stdio.h>
#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoRotation.h>


// My include files
#include "hyperboloid.h"
#include "ellipsoid.h"


class bishop
{
  SoMaterial *MAT;
 
  SoCylinder *BASE;
  SoGroup *BODY;
  SoCylinder *NECK;
  SoGroup *HEAD;
  SoSphere *BAUBLE;

  SoSeparator *BISHOP;

public:

  bishop(float red, float green, float blue);
  SoSeparator *getGeometry();
};


// Constructor

bishop::bishop(float red = 0.8, float green = 0.8, float blue = 0.0)
{

  MAT = new SoMaterial;
  MAT->diffuseColor.setValue(red, green, blue);
  MAT->shininess = 1.0;
  MAT->specularColor.setValue(0.8,0.8,0.8);

  
// Build the bishop base up

  int b = 6;  // b is the input b to the hyperboloid. 
              // Both cylinder and ellipsoid depend on its value.
  int a = 1;  // a is also an input to the hyperboloid
              // and the cylinder and ellipsoid depend on it as well,
              // however I will fix this as 1. and change b
              // depending on the piece type (pawn, rook etc.).

  int body_height = 14;


  BASE = new SoCylinder;
  BASE->radius = sqrt(1 + (body_height/1.3*body_height/1.3)/b*b);
                     // assuming a = 1
  BASE->height = (BASE->radius.getValue()) / 2.5;

  SoTransform *trans_board_to_base = new SoTransform;
  trans_board_to_base->translation.setValue(0,BASE->height.getValue()/2, 0);

  SoTransform *trans_base_to_body = new SoTransform;
  trans_base_to_body->translation = SbVec3f(0,(body_height/2 + (BASE->height.getValue())/2), 0);


  hyperboloid body(a,b,body_height);
  BODY = body.getGeometry(); 

  NECK = new SoCylinder;
  NECK->radius = BASE->radius;
  NECK->height = BASE->height;

  SoTransform *trans_body_to_neck = new SoTransform;
  trans_body_to_neck->translation = SbVec3f(0, (body_height/2 + (NECK->height.getValue())/2), 0);

// BISHOP head

// head_height depends on size of neck so as to be proportional
  int head_height =  (int) (2.5 * NECK->radius.getValue());

// ideally head_height should be even
  head_height += (head_height % 2);

// determine ellipsoid eccentricity by ea and eb which are a,b parameters 
// to the ellipsoid, noting that 2 * eb = head_height 

  float ea = 0.6;
  float eb = 0.5 * head_height;

  ellipsoid head(ea, eb, head_height);

  HEAD = head.getGeometry();

//  SoTransform *trans_neck_to_head = new SoTransform;
  //  trans_neck_to_head->translation = SbVec3f(0, (head_height/100.0 + (NECK->height.getValue())/2), 0); 

  BAUBLE = new SoSphere;
  BAUBLE->radius = (1.0/9.0) * (NECK->radius.getValue());

  SoTransform *trans_head_to_bauble = new SoTransform;
  trans_head_to_bauble->translation = SbVec3f(0, ((head_height/2) + (BAUBLE->radius.getValue()/2.0)), 0);



//Add the children to the root bishop node

  BISHOP = new SoSeparator;
  BISHOP->ref();

  BISHOP->addChild(MAT);


// The draggers move in the X-Y plane so the pieces must be centered around 
// the Z axis. Since my code is written to center it around the 
// Y axis, I apply a transform rotation to the scenegraph before drawing it

  SoRotation *rotate = new SoRotation;
  rotate->rotation.setValue(1,0,0,3.1416/3);
  BISHOP->addChild(rotate);

  BISHOP->addChild(trans_board_to_base);

  BISHOP->addChild(BASE);
  BISHOP->addChild(trans_base_to_body);
  BISHOP->addChild(BODY);
  BISHOP->addChild(trans_body_to_neck);
  BISHOP->addChild(NECK);
  //  BISHOP->addChild(trans_neck_to_head);
  BISHOP->addChild(HEAD);
  BISHOP->addChild(trans_head_to_bauble);
  BISHOP->addChild(BAUBLE);
} 

// getGeometry Member Function 

SoSeparator *bishop::getGeometry()
{
  return BISHOP;
}


/************************************************/
/* End of Code: Test Main program commented out */
/************************************************/  

/*
int main(int, char **argv)	{

// Initialize Inventor

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  bishop a(0.8, 0.8, 0);

// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("Madhulika");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/

#endif








