#ifndef _CHESS_BOARD_H
#define _CHESS_BOARD_H

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoCube.h>


class chessboard {
  int SIZE;
  
  SoSeparator *chessboard_root;

public:

  chessboard(int size, int flag_texture);
  SoSeparator *getGeometry();
};

// Constructor
chessboard::chessboard(int size = 40, int flag_texture = 0)
{
  
  SIZE = size;

  chessboard_root = new SoSeparator;
  chessboard_root->ref();

  if (flag_texture)            // Texture for black blocks if specified
    {
      SoTexture2 *black_texture = new SoTexture2;
      black_texture->filename.setValue("Darkwd.rgb");
      chessboard_root->addChild(black_texture);
    }
  else
    {
      SoMaterial *black_mat = new SoMaterial;
      black_mat->diffuseColor.setValue(0.0, 0.0, 0.0);
      chessboard_root->addChild(black_mat);
    }


  SoCube *block;
  SoTransform *trans;

  // Black blocks
  for(int j=0; j<8; j++)
    {  
      for (int i=0; i<4; i++)
	{
	  block = new SoCube;
	  block->height = SIZE;
	  block->width = SIZE;
	  block->depth = SIZE/2;
	  chessboard_root->addChild(block);
	  trans = new SoTransform;
	  trans->translation.setValue(SIZE*2,0,0);
	  chessboard_root->addChild(trans);
	}
      trans = new SoTransform;
      if (j%2)
	{
	  trans->translation.setValue(-9*SIZE,SIZE,0);
	  // Accounts for extra translation at the end of the loop
	}
      else
	{
	  trans->translation.setValue(-7*SIZE,SIZE,0);
	  // Accounts for extra translation at the end of the loop
	}
      chessboard_root->addChild(trans);
    }


  // White blocks
  trans = new SoTransform;  // Translate to position for white blocks
  trans->translation.setValue(SIZE,-8*SIZE,0);
  chessboard_root->addChild(trans);

  if (flag_texture)         // Texture for white blocks if specified
    {
      SoTexture2 *white_texture = new SoTexture2;
      white_texture->filename.setValue("Yew.rgb");
      chessboard_root->addChild(white_texture);
    }
  else
    {
      SoMaterial *white_mat = new SoMaterial;
      white_mat->diffuseColor.setValue(0.8, 0.8, 1.0);
      chessboard_root->addChild(white_mat);
    }


  for(j=0; j<8; j++)
    {  
      for (int i=0; i<4; i++)
	{
	  block = new SoCube;
	  block->height = SIZE;
	  block->width = SIZE;
	  block->depth = SIZE/2;
	  chessboard_root->addChild(block);
	  trans = new SoTransform;
	  trans->translation.setValue(SIZE*2,0,0);
	  chessboard_root->addChild(trans);
	}
      trans = new SoTransform;
      if (j%2)
	{
	  trans->translation.setValue(-7*SIZE,SIZE,0);
	  // Accounts for extra translation at the end of the loop
	}
      else
	{
	  trans->translation.setValue(-9*SIZE,SIZE,0);
	  //Accounts for extra translation at the end of the loop
	}
      chessboard_root->addChild(trans);
    }
}


SoSeparator *chessboard::getGeometry()
{
  return chessboard_root;
}

/******************************************/
/* End of Code. Test Program commented out*/
/******************************************/

/*
int main(int argc, char**argv)
{

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  chessboard a(40);


// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("ChessBoard");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/

#endif






