#ifndef _HYPER_H
#define _HYPER_H


#include <stdio.h>
#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/SbLinear.h>


class hyperboloid 
{
  int A;
  int B;
  int HEIGHT;

  SoGroup *hyperboloid_root;

public:

  hyperboloid(int a, int b, int height);
  SoGroup *getGeometry();
};



hyperboloid::hyperboloid(int a = 1, int b = 6, int height = 20)
{

// Default is a rook

  double PI = 3.1416;

  A = a;
  B = b;
  HEIGHT = height;

  SbVec3f coords3d[1000];
  int ctr = 0;
  int res = 20;

// Note this class is for my use and convenience only, therefore
// it will be my responsibiltiy to send decent values for a and b, in 
// particular non zero values for b.
// A and B represent a and b in the standard Hyperbola equation.


  for (int y = (-1 * HEIGHT/2); y <= HEIGHT/2; y++)
    {
      double rad = sqrt((a*a*y*y + a*a*b*b)/b*b);

      // I have arbitrarily decided that the resolution for the
      // concentric rows of points to be quadmeshed will be 20
      // this can be changed by setting the res variable upstairs

      for (int i=0; i<=res; i++)
	{
	  double theta = (2*PI/res)*i;
	  double x = rad * cos(theta);
	  double z = rad * sin(theta);
	  coords3d[ctr++] = SbVec3f(x,y,z);
	}
    }


  SoCoordinate3 *coord = new SoCoordinate3;
  coord->point.setValues(0, ((res+1) * (HEIGHT+1)) , coords3d);

  SoQuadMesh *hyper_quad = new SoQuadMesh;
  hyper_quad->verticesPerRow = res + 1;
  hyper_quad->verticesPerColumn = height + 1;

  SoShapeHints *hints = new SoShapeHints;
  hints->creaseAngle = 0.6; 

  hyperboloid_root = new SoGroup;
  hyperboloid_root->ref();

  hyperboloid_root->addChild(hints);
  hyperboloid_root->addChild(coord);
  hyperboloid_root->addChild(hyper_quad);
}


SoGroup *hyperboloid::getGeometry()
{
  return hyperboloid_root;
}


/*********************************************************/
/*   COMMENTED OUT AFTER THIS... END OF CODE :)  ....   **/         
/*********************************************************/

/*************************************
 * TEST main() for hyperboloid class *
 *************************************/

/*
int main(int, char **argv)	{

// Initialize Inventor

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  hyperboloid a(1,6,20);

// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("Madhulika");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/


#endif
