#ifndef _KNIGHT_H
#define _KNIGHT_H

#include <stdio.h>
#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoRotation.h>


// My include files
#include "hyperboloid.h"
#include "ellipsoid.h"
#include "knight_head.h"


class knight
{
  SoMaterial *MAT;
 
  SoCylinder *BASE;
  SoGroup *BODY;
  SoCylinder *NECK;
  SoGroup *HEAD;

  SoSeparator *KNIGHT;

public:

  knight(float red, float green, float blue);
  SoSeparator *getGeometry();
};


// Constructor

knight::knight(float red = 0.8, float green = 0.8, float blue = 0.0)
{

  MAT = new SoMaterial;
  MAT->diffuseColor.setValue(red, green, blue);
  MAT->shininess = 1.0;
  MAT->specularColor.setValue(0.8,0.8,0.8);


// Build the knight base up

  int b = 5;  // b is the input b to the hyperboloid. 
              // Both cylinder and head depend on its value.
  int a = 1;  // a is also an input to the hyperboloid
              // and the cylinder and head depend on it as well,
              // however I will fix this as 1. and change b
              // depending on the piece type (pawn, rook etc.).

  int body_height = 12;


  BASE = new SoCylinder;
  BASE->radius = sqrt(1 + (body_height/1.3*body_height/1.3)/b*b);
                     // assuming a = 1
  BASE->height = (BASE->radius.getValue()) / 2.5;

  SoTransform *trans_board_to_base = new SoTransform;
  trans_board_to_base->translation.setValue(0,BASE->height.getValue()/2, 0);

  SoTransform *trans_base_to_body = new SoTransform;
  trans_base_to_body->translation = SbVec3f(0,(body_height/2 + (BASE->height.getValue())/2), 0);

  hyperboloid body(a,b,body_height);
  BODY = body.getGeometry(); 

  NECK = new SoCylinder;
  NECK->radius = BASE->radius;
  NECK->height = BASE->height;

  SoTransform *trans_body_to_neck = new SoTransform;
  trans_body_to_neck->translation = SbVec3f(0, (body_height/2 + (NECK->height.getValue())/2), 0);

// KNIGHT head

  float knight_head_height = body_height;
  float knight_head_width = NECK->radius.getValue();
  float knight_head_depth = (1.0/3.0) * NECK->radius.getValue();

  knight_head head(knight_head_height, knight_head_width, knight_head_depth);
  HEAD = head.getGeometry();

  // Set translation to be bottom front left corner of knight head
  SoTransform *trans_neck_to_head = new SoTransform;
  trans_neck_to_head->translation = SbVec3f(-1*knight_head_width/2, NECK->height.getValue()/2, -1*knight_head_depth/2); 


//Add the children to the root knight node

  KNIGHT = new SoSeparator;
  KNIGHT->ref();

  KNIGHT->addChild(MAT);

// The draggers move in the X-Y plane so the pieces must be centered around 
// the Z axis. Since my code is written to center it around the 
// Y axis, I apply a transform rotation to the scenegraph before drawing it

  SoRotation *rotate = new SoRotation;
  rotate->rotation.setValue(1,0,0,3.1416/3);
  KNIGHT->addChild(rotate);

  KNIGHT->addChild(trans_board_to_base);

  KNIGHT->addChild(BASE);
  KNIGHT->addChild(trans_base_to_body);
  KNIGHT->addChild(BODY);
  KNIGHT->addChild(trans_body_to_neck);
  KNIGHT->addChild(NECK);
  KNIGHT->addChild(trans_neck_to_head);
  KNIGHT->addChild(HEAD);

} 

// getGeometry Member Function 

SoSeparator *knight::getGeometry()
{
  return KNIGHT;
}


/************************************************/
/* End of Code: Test Main program commented out */
/************************************************/  

/*
int main(int, char **argv)	{

// Initialize Inventor

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  knight a(0.8, 0.8, 0);

// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("Madhulika");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/

#endif








