#ifndef _QUEEN_H
#define _QUEEN_H

#include <stdio.h>
#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoRotation.h>


// My include files
#include "hyperboloid.h"

class queen
{
  SoMaterial *MAT;
 
  SoCylinder *BASE;
  SoGroup *BODY;
  SoCylinder *NECK;
  SoSphere *HEAD;
  SoSphere *BAUBLE;

  SoSeparator *QUEEN;

public:

  queen(float red, float green, float blue);
  SoSeparator *getGeometry();
};


// Constructor

queen::queen(float red = 0.8, float green = 0.8, float blue = 0.0)
{

  MAT = new SoMaterial;
  MAT->diffuseColor.setValue(red, green, blue);
  MAT->shininess = 1.0;
  MAT->specularColor.setValue(0.8,0.8,0.8);
  
// Build the queen base up

  int b = 6;  // b is the input b to the hyperboloid. 
              // Both cylinder and sphere depend on its value.
  int a = 1;  // a is also an input to the hyperboloid
              // and the bylinder and sphere depend on it as well,
              // however I will fix this as 1. and change b
              // depending on the piece type (pawn, rook etc.).

  int body_height = 22;


  BASE = new SoCylinder;
  BASE->radius = sqrt(1 + (body_height/1.5*body_height/1.5)/b*b);
                     // assuming a = 1
  BASE->height = (BASE->radius.getValue()) / 2.5;

  SoTransform *trans_board_to_base = new SoTransform;
  trans_board_to_base->translation.setValue(0,BASE->height.getValue()/2, 0);

  SoTransform *trans_base_to_body = new SoTransform;
  trans_base_to_body->translation = SbVec3f(0,(body_height/2 + (BASE->height.getValue())/2), 0);


  hyperboloid body(a,b,body_height);
  BODY = body.getGeometry(); 

  NECK = new SoCylinder;
  NECK->radius = BASE->radius;
  NECK->height = BASE->height;

  SoTransform *trans_body_to_neck = new SoTransform;
  trans_body_to_neck->translation = SbVec3f(0, (body_height/2 + (NECK->height.getValue())/2), 0);

  HEAD = new SoSphere;
  HEAD->radius = ((3.5/5.0) * (NECK->radius.getValue()));

  SoTransform *trans_neck_to_head = new SoTransform;
  trans_neck_to_head->translation = SbVec3f(0, ((HEAD->radius.getValue())/1.8 - (NECK->height.getValue())/2), 0); 

  BAUBLE = new SoSphere;
  BAUBLE->radius = (1.0/6.0) * (HEAD->radius.getValue());

  SoTransform *trans_head_to_bauble = new SoTransform;
  trans_head_to_bauble->translation = SbVec3f(0, ((HEAD->radius.getValue()) + (BAUBLE->radius.getValue()/1.5)), 0);



//Add the children to the root queen node

  QUEEN = new SoSeparator;
  QUEEN->ref();

  QUEEN->addChild(MAT);

// The draggers move in the X-Y plane so the pieces must be centered around 
// the Z axis. Since my code is written to center it around the 
// Y axis, I apply a transform rotation to the scenegraph before drawing it

  SoRotation *rotate = new SoRotation;
  rotate->rotation.setValue(1,0,0,3.1416/3);
  QUEEN->addChild(rotate);

  QUEEN->addChild(trans_board_to_base);
  QUEEN->addChild(BASE);
  QUEEN->addChild(trans_base_to_body);
  QUEEN->addChild(BODY);
  QUEEN->addChild(trans_body_to_neck);
  QUEEN->addChild(NECK);

// THE CROWN

  float PI = 3.1416;
  float theta;
  float virtual_rad;
  SoTransform* trans_neck_to_cone;
  SoTransform* trans_cone_to_neck;
  SoCone *cone;

  for (int i = 0; i<30; i++)
    {
      theta = 2*PI*i/30.0;

      cone = new SoCone;
      cone->bottomRadius = 0.5*2*PI*(NECK->radius.getValue())/30.0;
      cone->height = 1.7 * cone->bottomRadius.getValue();

      virtual_rad = NECK->radius.getValue() - cone->bottomRadius.getValue();

      trans_neck_to_cone = new SoTransform;
      trans_neck_to_cone->translation = SbVec3f(virtual_rad*cos(theta), NECK->height.getValue()/2 + cone->height.getValue()/2, virtual_rad*sin(theta));

      trans_cone_to_neck = new SoTransform;
      trans_cone_to_neck->translation = SbVec3f(-1*virtual_rad*cos(theta), -1*NECK->height.getValue()/2-cone->height.getValue()/2, -1*virtual_rad*sin(theta));

      QUEEN->addChild(trans_neck_to_cone);
      QUEEN->addChild(cone);
      QUEEN->addChild(trans_cone_to_neck);
    }

  QUEEN->addChild(trans_neck_to_head);
  QUEEN->addChild(HEAD);
  QUEEN->addChild(trans_head_to_bauble);
  QUEEN->addChild(BAUBLE);
} 

// getGeometry Member Function 

SoSeparator *queen::getGeometry()
{
  return QUEEN;
}


/************************************************/
/* End of Code: Test Main program commented out */
/************************************************/  

/*
int main(int, char **argv)	{

// Initialize Inventor

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  queen a(0.8, 0.8, 0);

// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("Madhulika");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/

#endif








