#ifndef _ROOK_H
#define _ROOK_H

#include <stdio.h>
#include <math.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoFaceSet.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoRotation.h>


// My include files
#include <hyperboloid.h>

class rook
{
  SoMaterial *MAT;
 
  SoCylinder *BASE;
  SoGroup *BODY;
  SoCylinder *NECK;
  SoCoordinate3 *(coords3d[10]);
  SoFaceSet *(MOAT[10]);

  SoSeparator *ROOK;

public:

  rook(float red, float green, float blue);
  SoSeparator *getGeometry();
};


// Constructor

rook::rook(float red = 0.8, float green = 0.8, float blue = 0.0)
{

  MAT = new SoMaterial;
  MAT->diffuseColor.setValue(red, green, blue);
  MAT->shininess = 1.0;
  MAT->specularColor.setValue(0.8,0.8,0.8);
  
// Build the rook base up

  int b = 8;  // b is the input b to the hyperboloid. 

  int a = 1;  // a is also an input to the hyperboloid
              // and the cylinder and moat depend on it as well,
              // however I will fix this as 1. and change b
              // depending on the piece type (pawn, rook etc.).

  int body_height = 16;

  BASE = new SoCylinder;
  BASE->radius = sqrt(1 + (body_height*body_height)/b*b);
                     // assuming a = 1
  BASE->height = (BASE->radius.getValue()) / 2.5;

  SoTransform *trans_board_to_base = new SoTransform;
  trans_board_to_base->translation.setValue(0,BASE->height.getValue()/2, 0);

  SoTransform *trans_base_to_body = new SoTransform;
  trans_base_to_body->translation = SbVec3f(0,(body_height/2 + (BASE->height.getValue())/2), 0);

  hyperboloid body(a,b,body_height);
  BODY = body.getGeometry(); 

  NECK = new SoCylinder;
  NECK->radius = BASE->radius;
  NECK->height = BASE->height;

  SoTransform *trans_body_to_neck = new SoTransform;
  trans_body_to_neck->translation = SbVec3f(0, (body_height/2 + (NECK->height.getValue())/2), 0);

// Moat of the rook head, each piece will be a FaceSet

  SbVec3f coords_arr[20];

  float R = NECK->radius.getValue();
  float R_dec = (1.0/10.0) * (NECK->radius.getValue());
  float y_inc = (3.0/5.0) * (NECK->height.getValue());

  float theta;
  float theta_next;

  int vert_arr[5];
  for (int k=0; k<5; k++)
     vert_arr[k] = 4;

  float PI = 3.1416;

  for (int i=0; i<10; i++)
    {
      MOAT[i] = new SoFaceSet;
      coords3d[i] = new SoCoordinate3;

      theta = (2*PI/20)*(2*i);       
      theta_next = (2*PI/20)*(2*i + 1);

      coords_arr[0] = SbVec3f((R-R_dec)*cos(theta), 0, (R-R_dec)*sin(theta));
      coords_arr[1] = SbVec3f((R)*cos(theta), 0, (R)*sin(theta));
      coords_arr[2] = SbVec3f((R)*cos(theta), y_inc, (R)*sin(theta));
      coords_arr[3] = SbVec3f((R-R_dec)*cos(theta), y_inc, (R-R_dec)*sin(theta));

      coords_arr[4] = SbVec3f((R)*cos(theta), 0, (R)*sin(theta));
      coords_arr[5] = SbVec3f((R)*cos(theta_next), 0, (R)*sin(theta_next));
      coords_arr[6] = SbVec3f((R)*cos(theta_next), y_inc, (R)*sin(theta_next));
      coords_arr[7] = SbVec3f((R)*cos(theta), y_inc, (R)*sin(theta));

      coords_arr[8] = SbVec3f((R)*cos(theta_next), 0, (R)*sin(theta_next));
      coords_arr[9] = SbVec3f((R-R_dec)*cos(theta_next), 0, (R-R_dec)*sin(theta_next));
      coords_arr[10] = SbVec3f((R-R_dec)*cos(theta_next), y_inc, (R-R_dec)*sin(theta_next));
      coords_arr[11] = SbVec3f((R)*cos(theta_next), y_inc, (R)*sin(theta_next));

      coords_arr[12] = SbVec3f((R-R_dec)*cos(theta_next), 0, (R-R_dec)*sin(theta_next));
      coords_arr[13] = SbVec3f((R-R_dec)*cos(theta), 0, (R-R_dec)*sin(theta));
      coords_arr[14] = SbVec3f((R-R_dec)*cos(theta), y_inc, (R-R_dec)*sin(theta));
      coords_arr[15] = SbVec3f((R-R_dec)*cos(theta_next), y_inc, (R-R_dec)*sin(theta_next));

      coords_arr[16] = SbVec3f((R-R_dec)*cos(theta), y_inc, (R-R_dec)*sin(theta));
      coords_arr[17] = SbVec3f((R)*cos(theta), y_inc, (R)*sin(theta));
      coords_arr[18] = SbVec3f((R)*cos(theta_next), y_inc, (R)*sin(theta_next));
      coords_arr[19] = SbVec3f((R-R_dec)*cos(theta_next), y_inc, (R-R_dec)*sin(theta_next));


// Set the coordinate state for the FaceSet node in MOAT[i]
      coords3d[i]->point.setValues(0,20,coords_arr);
// Set parameters in the FaceSet node
      MOAT[i]->numVertices.setValues(0,5,vert_arr);

    }
      

  SoTransform *trans_neck_to_moat = new SoTransform;
  trans_neck_to_moat->translation = SbVec3f(0, (NECK->height.getValue())/2, 0);

  SoShapeHints *hints = new SoShapeHints;
  hints->vertexOrdering.setValue(SoShapeHints::CLOCKWISE);

//Add the children to the root rook node

  ROOK = new SoSeparator;
  ROOK->ref();
  ROOK->addChild(MAT);


// The draggers move in the X-Y plane so the pieces must be centered around 
// the Z axis. Since my code is written to center it around the 
// Y axis, I apply a transform rotation to the scenegraph before drawing it

  SoRotation *rotate = new SoRotation;
  rotate->rotation.setValue(1,0,0,3.1416/3);
  ROOK->addChild(rotate);

  ROOK->addChild(trans_board_to_base);
  ROOK->addChild(BASE);
  ROOK->addChild(trans_base_to_body);
  ROOK->addChild(BODY);
  ROOK->addChild(trans_body_to_neck);
  ROOK->addChild(NECK);
  ROOK->addChild(trans_neck_to_moat);
  ROOK->addChild(hints);

  for(int j=0; j<10; j++)
    {
      ROOK->addChild(coords3d[j]);
      ROOK->addChild(MOAT[j]);
    }
} 

// getGeometry Member Function 

SoSeparator *rook::getGeometry()
{
  return ROOK;
}


/************************************************/
/* End of Code: Test Main program commented out */
/************************************************/  

/*
int main(int, char **argv)	{

// Initialize Inventor

  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1);

  rook a(0.8, 0.8, 0);

// Set up viewer / Render area to see the scene graph

  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(myWindow);
  viewer->setSceneGraph(a.getGeometry());
  viewer->setTitle("Madhulika");
  viewer->show();


  SoXt::show(myWindow);
  SoXt::mainLoop();
}
*/

#endif


















